/*
 * JORAM: Java(TM) Open Reliable Asynchronous Messaging
 * Copyright (C) 2015 - 2024 ScalAgent Distributed Technologies
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): 
 */
package joram.jms2;

import jakarta.jms.ConnectionFactory;
import jakarta.jms.JMSContext;
import jakarta.jms.JMSRuntimeException;

import org.ow2.joram.jakarta.jms.admin.AdminModule;
import org.ow2.joram.jakarta.jms.admin.User;
import org.ow2.joram.jakarta.jms.tcp.TcpConnectionFactory;

import framework.TestCase;

/**
 * Creates a JMSContext and close it to verify that the connection is correctly closed.
 */
public class Test0 extends TestCase {
  public static void main(String[] args) {
    new Test0().run();
  }

  public void run()  {
    try {
      startAgentServer((short) 0);
      Thread.sleep(1000);
      
      ConnectionFactory cf = TcpConnectionFactory.create("localhost", 2560);
      AdminModule.connect(cf, "root", "root");   
      User.create("anonymous", "anonymous", 0);
      AdminModule.disconnect();

      JMSContext context = cf.createContext();
      context.close();
      
      boolean isexc = false;
      try {
        context.start();
      } catch (JMSRuntimeException exc) {
        exc.printStackTrace();
        isexc = true;
      }
      assertTrue("JMSContext connection is not correctly closed.", isexc);

      context = cf.createContext();
      JMSContext context1 = context.createContext(0);
      context.close();
      
      isexc = false;
      try {
        context.start();
      } catch (JMSRuntimeException exc) {
        exc.printStackTrace();
        isexc = true;
      }
      assertFalse("JMSContext connection should not be closed.", isexc);

      context1.close();
      isexc = false;
      try {
        context.start();
      } catch (JMSRuntimeException exc) {
        exc.printStackTrace();
        isexc = true;
      }
      assertTrue("JMSContext connection is not correctly closed.", isexc);
      
    } catch (Throwable exc) {
      exc.printStackTrace();
      error(exc);
    } finally {
      stopAgentServer((short) 0);
      endTest();     
    }
  }
}
