/*
 * JORAM: Java(TM) Open Reliable Asynchronous Messaging
 * Copyright (C) 2007 - 2010 ScalAgent Distributed Technologies
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s):Badolle Fabien (ScalAgent D.T.)
 * Contributor(s): 
 */
package joram.ssl;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.Topic;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.admin.User;
import org.objectweb.joram.client.jms.tcp.TcpConnectionFactory;

import framework.TestCase;

/**
 * Test :
 */
public class Test extends TestCase {

  public static void main(String[] args) {
    new Test().run();
  }

  public void run() {
    try {
      System.out.println("server start");
      
      String serverks = System.getProperty("server.ks", "joram_ks");
      String serverpass = System.getProperty("server.pass", "jorampass");
      String clientauth = System.getProperty("client.auth", "NEED");
      startAgentServer((short) 0,
                       new String[] {"-Dorg.objectweb.joram.clientAuth=" + clientauth,
                                     "-Dorg.objectweb.joram.keystore=" + serverks,
                                     "-Dorg.objectweb.joram.keystorepass=" + serverpass});
      Thread.sleep(1000);

      // Joram client and server use the same property names, so we have to overload these
      // properties prior to create the Connection.
      String clientks = System.getProperty("client.ks");
      if (clientks != null)
        System.setProperty("org.objectweb.joram.keystore", clientks);
      String clientpass = System.getProperty("client.pass");
      if (clientpass != null)
        System.setProperty("org.objectweb.joram.keystorepass", clientpass);

      System.out.println("ks=" + clientks + ", pass=" + clientpass);
      
      admin();
      System.out.println("admin config ok");

      Context ictx = new InitialContext();
      Queue queue = (Queue) ictx.lookup("queue");
      ConnectionFactory cf = (ConnectionFactory) ictx.lookup("cf");
      ictx.close();

      Connection cnx = cf.createConnection();
      Session sessionp = cnx.createSession(false, Session.AUTO_ACKNOWLEDGE);
      Session sessionc = cnx.createSession(false, Session.AUTO_ACKNOWLEDGE);
      cnx.start();

      // create a producer and a consumer
      MessageProducer producer = sessionp.createProducer(queue);
      MessageConsumer consumer = sessionc.createConsumer(queue);
      // create a text message send to the queue by the producer 
      TextMessage msg = sessionp.createTextMessage();
      msg.setText("message_type_text");
      producer.send(msg);
      // the consumer receive the message from the queue
      Message msg1 = consumer.receive();
      TextMessage msg2 = (TextMessage) msg1;

      //test messages
      assertEquals(msg.getJMSMessageID(), msg1.getJMSMessageID());
      assertEquals(msg.getJMSType(), msg1.getJMSType());
      assertEquals(msg.getJMSDestination(), msg1.getJMSDestination());
      assertEquals("message_type_text", msg2.getText());

      cnx.close();
    } catch (Throwable exc) {
      exc.printStackTrace();
      error(exc);
    } finally {
      System.out.println("Server stop ");
      killAgentServer((short) 0);
      endTest();
    }
  }

  public void admin() throws Exception {

    AdminModule.connect("root", "root", 30, "org.objectweb.joram.client.jms.tcp.ReliableSSLTcpClient");

    Queue queue = Queue.create("queue");
    Topic topic = Topic.create("topic");

    User.create("anonymous", "anonymous");

    queue.setFreeReading();
    topic.setFreeReading();
    queue.setFreeWriting();
    topic.setFreeWriting();

    javax.jms.ConnectionFactory cf = TcpConnectionFactory.create("localhost", 16010,
        "org.objectweb.joram.client.jms.tcp.ReliableSSLTcpClient");

    javax.naming.Context jndiCtx = new javax.naming.InitialContext();
    jndiCtx.bind("cf", cf);
    jndiCtx.bind("queue", queue);
    jndiCtx.bind("topic", topic);
    jndiCtx.close();

    AdminModule.disconnect();
  }
}
